@@ -17,12 +17,15 @@ import salt.utils.url
 import salt.crypt
 import salt.transport
 from salt.exceptions import CommandExecutionError
+from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=import-error,no-name-in-module
 
 # Import 3rd-party libs
 import salt.ext.six as six
 
 log = logging.getLogger(__name__)
 
+__proxyenabled__ = ['*']
+
 
 def _auth():
     '''
@@ -82,12 +85,24 @@ def recv(files, dest):
 def _mk_client():
     '''
     Create a file client and add it to the context.
+
+    Each file client needs to correspond to a unique copy
+    of the opts dictionary, therefore it's hashed by the
+    id of the __opts__ dict
     '''
-    if 'cp.fileclient' not in __context__:
-        __context__['cp.fileclient'] = \
+    if 'cp.fileclient_{0}'.format(id(__opts__)) not in __context__:
+        __context__['cp.fileclient_{0}'.format(id(__opts__))] = \
                 salt.fileclient.get_file_client(__opts__)
 
 
+def _client():
+    '''
+    Return a client, hashed by the list of masters
+    '''
+    _mk_client()
+    return __context__['cp.fileclient_{0}'.format(id(__opts__))]
+
+
 def _render_filenames(path, dest, saltenv, template, **kw):
     '''
     Process markup in the :param:`path` and :param:`dest` variables (NOT the
@@ -199,8 +214,7 @@ def get_file(path,
     if not hash_file(path, saltenv):
         return ''
     else:
-        _mk_client()
-        return __context__['cp.fileclient'].get_file(
+        return _client().get_file(
                 path,
                 dest,
                 makedirs,
@@ -235,7 +249,6 @@ def get_template(path,
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
     if 'salt' not in kwargs:
         kwargs['salt'] = __salt__
     if 'pillar' not in kwargs:
@@ -244,7 +257,7 @@ def get_template(path,
         kwargs['grains'] = __grains__
     if 'opts' not in kwargs:
         kwargs['opts'] = __opts__
-    return __context__['cp.fileclient'].get_template(
+    return _client().get_template(
             path,
             dest,
             template,
@@ -276,8 +289,7 @@ def get_dir(path, dest, saltenv='base', template=None, gzip=None, env=None, **kw
 
     (path, dest) = _render_filenames(path, dest, saltenv, template, **kwargs)
 
-    _mk_client()
-    return __context__['cp.fileclient'].get_dir(path, dest, saltenv, gzip)
+    return _client().get_dir(path, dest, saltenv, gzip)
 
 
 def get_url(path, dest, saltenv='base', env=None):
@@ -304,11 +316,10 @@ def get_url(path, dest, saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
     if dest:
-        return __context__['cp.fileclient'].get_url(path, dest, False, saltenv)
+        return _client().get_url(path, dest, False, saltenv)
     else:
-        return __context__['cp.fileclient'].get_url(path, None, False, saltenv, no_cache=True)
+        return _client().get_url(path, None, False, saltenv, no_cache=True)
 
 
 def get_file_str(path, saltenv='base', env=None):
@@ -356,19 +367,40 @@ def cache_file(path, saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
+    contextkey = '{0}_|-{1}_|-{2}'.format('cp.cache_file', path, saltenv)
+    path_is_remote = _urlparse(path).scheme in ('http', 'https', 'ftp')
+    try:
+        if path_is_remote and contextkey in __context__:
+            # Prevent multiple caches in the same salt run. Affects remote URLs
+            # since the master won't know their hash, so the fileclient
+            # wouldn't be able to prevent multiple caches if we try to cache
+            # the remote URL more than once.
+            if os.path.isfile(__context__[contextkey]):
+                return __context__[contextkey]
+            else:
+                # File is in __context__ but no longer exists in the minion
+                # cache, get rid of the context key and re-cache below.
+                # Accounts for corner case where file is removed from minion
+                # cache between cp.cache_file calls in the same salt-run.
+                __context__.pop(contextkey)
+    except AttributeError:
+        pass
 
     path, senv = salt.utils.url.split_env(path)
     if senv:
         saltenv = senv
 
-    result = __context__['cp.fileclient'].cache_file(path, saltenv)
+    result = _client().cache_file(path, saltenv)
     if not result:
         log.error(
             'Unable to cache file {0!r} from saltenv {1!r}.'.format(
                 path, saltenv
             )
         )
+    if path_is_remote:
+        # Cache was successful, store the result in __context__ to prevent
+        # multiple caches (see above).
+        __context__[contextkey] = result
     return result
 
 
@@ -393,8 +425,7 @@ def cache_files(paths, saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].cache_files(paths, saltenv)
+    return _client().cache_files(paths, saltenv)
 
 
 def cache_dir(path, saltenv='base', include_empty=False, include_pat=None,
@@ -439,8 +470,7 @@ def cache_dir(path, saltenv='base', include_empty=False, include_pat=None,
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].cache_dir(
+    return _client().cache_dir(
         path, saltenv, include_empty, include_pat, exclude_pat
     )
 
@@ -464,8 +494,7 @@ def cache_master(saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].cache_master(saltenv)
+    return _client().cache_master(saltenv)
 
 
 def cache_local_file(path):
@@ -492,8 +521,7 @@ def cache_local_file(path):
             return path_cached
 
     # The file hasn't been cached or has changed; cache it
-    _mk_client()
-    return __context__['cp.fileclient'].cache_local_file(path)
+    return _client().cache_local_file(path)
 
 
 def list_states(saltenv='base', env=None):
@@ -515,8 +543,7 @@ def list_states(saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].list_states(saltenv)
+    return _client().list_states(saltenv)
 
 
 def list_master(saltenv='base', prefix='', env=None):
@@ -538,8 +565,7 @@ def list_master(saltenv='base', prefix='', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].file_list(saltenv, prefix)
+    return _client().file_list(saltenv, prefix)
 
 
 def list_master_dirs(saltenv='base', prefix='', env=None):
@@ -561,8 +587,7 @@ def list_master_dirs(saltenv='base', prefix='', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].dir_list(saltenv, prefix)
+    return _client().dir_list(saltenv, prefix)
 
 
 def list_master_symlinks(saltenv='base', prefix='', env=None):
@@ -584,8 +609,7 @@ def list_master_symlinks(saltenv='base', prefix='', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].symlink_list(saltenv, prefix)
+    return _client().symlink_list(saltenv, prefix)
 
 
 def list_minion(saltenv='base', env=None):
@@ -607,8 +631,7 @@ def list_minion(saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].file_local_list(saltenv)
+    return _client().file_local_list(saltenv)
 
 
 def is_cached(path, saltenv='base', env=None):
@@ -631,8 +654,7 @@ def is_cached(path, saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].is_cached(path, saltenv)
+    return _client().is_cached(path, saltenv)
 
 
 def hash_file(path, saltenv='base', env=None):
@@ -656,8 +678,11 @@ def hash_file(path, saltenv='base', env=None):
         # Backwards compatibility
         saltenv = env
 
-    _mk_client()
-    return __context__['cp.fileclient'].hash_file(path, saltenv)
+    path, senv = salt.utils.url.split_env(path)
+    if senv:
+        saltenv = senv
+
+    return _client().hash_file(path, saltenv)
 
 
 def push(path, keep_symlinks=False, upload_path=None):
