@@ -159,7 +159,10 @@ def _get_csr_extensions(csr):
     csrtempfile.flush()
     csryaml = _parse_openssl_req(csrtempfile.name)
     csrtempfile.close()
-    csrexts = csryaml['Certificate Request']['Data']['Requested Extensions']
+    try:
+        csrexts = csryaml['Certificate Request']['Data']['Requested Extensions']
+    except TypeError:
+        csrexts = {}
 
     for short_name, long_name in six.iteritems(EXT_NAME_MAPPINGS):
         if long_name in csrexts:
@@ -256,7 +259,7 @@ def _dec2hex(decval):
     '''
     Converts decimal values to nicely formatted hex strings
     '''
-    return _pretty_hex('{:X}'.format(decval))
+    return _pretty_hex('{0:X}'.format(decval))
 
 
 def _text_or_file(input_):
@@ -639,7 +642,7 @@ def create_private_key(path=None, text=False, bits=2048):
         If ``True``, return the PEM text without writing to a file. Default ``False``.
 
     bits:
-        Lenth of the private key in bits. Default 2048
+        Length of the private key in bits. Default 2048
 
     CLI Example:
 
